@@ -129,8 +129,25 @@ char *concat(const char *a, int b) {
  * Runs the given executable, returning its error code.
  */
 int execute(const std::string &exePath, const char **args) {
+#if LDC_LLVM_VER >= 700
+  std::vector<llvm::StringRef> argv;
+  for (auto pArg = args; *pArg; ++pArg) {
+    argv.push_back(*pArg);
+  }
+  auto envVars = llvm::None;
+#else
+  auto argv = args;
+  auto envVars = nullptr;
+#endif
+
   std::string errorMsg;
-  int rc = ls::ExecuteAndWait(exePath, args, nullptr, nullptr, 0, 0, &errorMsg);
+  int rc = ls::ExecuteAndWait(exePath, argv, envVars,
+#if LDC_LLVM_VER >= 600
+                              {},
+#else
+                              nullptr,
+#endif
+                              0, 0, &errorMsg);
   if (!errorMsg.empty()) {
     error("Error executing %s: %s", exePath.c_str(), errorMsg.c_str());
   }
