@@ -47,7 +47,9 @@
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Target/TargetMachine.h"
-#if LDC_LLVM_VER >= 306
+#if LDC_LLVM_VER >= 600
+#include "llvm/CodeGen/TargetSubtargetInfo.h"
+#elif LDC_LLVM_VER >= 306
 #include "llvm/Target/TargetSubtargetInfo.h"
 #endif
 #include "llvm/LinkAllIR.h"
@@ -140,7 +142,11 @@ void printVersion(llvm::raw_ostream &OS) {
   // redirecting stdout to a file.
   OS.flush();
 
-  llvm::TargetRegistry::printRegisteredTargetsForVersion();
+  llvm::TargetRegistry::printRegisteredTargetsForVersion(
+#if LDC_LLVM_VER >= 600
+    OS
+#endif
+    );
 
   exit(EXIT_SUCCESS);
 }
@@ -342,7 +348,11 @@ static void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
 
   final_args.insert(final_args.end(), &argv[1], &argv[argc]);
 
+#if LDC_LLVM_VER >= 600
+  cl::SetVersionPrinter(&printVersion);
+#else
   cl::SetVersionPrinter(&printVersionStdout);
+#endif
 
   hideLLVMOptions();
   cl::ParseCommandLineOptions(final_args.size(),
@@ -692,6 +702,7 @@ static void registerPredefinedTargetVersions() {
   case llvm::Triple::mips:
   case llvm::Triple::mipsel:
     VersionCondition::addPredefinedGlobalIdent("MIPS");
+    VersionCondition::addPredefinedGlobalIdent("MIPS32");
     registerPredefinedFloatABI("MIPS_SoftFloat", "MIPS_HardFloat");
     registerMipsABI();
     break;
